@@ -1,30 +1,8 @@
 /*
- * Copyright (c) 1999-2009 Douglas Gilbert.
+ * Copyright (c) 1999-2011 Douglas Gilbert.
  * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
+ * Use of this source code is governed by a BSD-style
+ * license that can be found in the BSD_LICENSE file.
  */
 
 /*
@@ -44,11 +22,15 @@
 #include "sg_cmds_basic.h"
 #include "sg_pt.h"
 
-
-static char * version_str = "1.45 20090228";
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 
-#define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
+static char * version_str = "1.55 20110730";
+
+
+#define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
 #define EBUFF_SZ 256
 
 #define DEF_PT_TIMEOUT 60       /* 60 seconds */
@@ -124,44 +106,63 @@ sg_cmds_close_device(int device_fd)
 }
 
 
-/* Returns -2 for sense data (may not be fatal), -1 for failed or the
-   number of bytes fetched. For data out (to device) or no data, set
-   'mx_resp_len' to 0 or less. If -2 returned then sense category
-   output via 'o_sense_cat' pointer (if not NULL). Outputs to
-   sg_warnings_strm (def: stderr) if problems; depending on 'noisy'
-   and 'verbose' */
+/* This is a helper function used by sg_cmds_* implementations after
+ * the call to the pass-through. pt_res is returned from do_scsi_pt().
+ * If valid sense data is found it is decoded and output to sg_warnings_strm
+ * (def: stderr); depending on the 'noisy' and 'verbose' settings.
+ * Returns -2 for sense data (may not be fatal), -1 for failed, or the
+ * number of data in bytes received. For data out (to device) or no data,
+ * set 'mx_di_len' to 0 or less. If -2 returned then sense category
+ * output via 'o_sense_cat' pointer (if not NULL). Note that several sense
+ * categories also have data in bytes received; -2 is still returned. */
 int
-sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin, int res,
-                     int mx_resp_len, const unsigned char * sense_b,
+sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
+                     int pt_res, int mx_di_len, const unsigned char * sbp,
                      int noisy, int verbose, int * o_sense_cat)
 {
-    int got, cat, duration, slen, scat, n, resid;
+    int got, cat, duration, slen, scat, n, resid, resp_code;
+    int check_data_in = 0;
     char b[1024];
 
     if (NULL == leadin)
         leadin = "";
-    if (res < 0) {
+    if (pt_res < 0) {
         if (noisy || verbose)
             fprintf(sg_warnings_strm, "%s: pass through os error: %s\n",
-                    leadin, safe_strerror(-res));
+                    leadin, safe_strerror(-pt_res));
         return -1;
-    } else if (SCSI_PT_DO_BAD_PARAMS == res) {
+    } else if (SCSI_PT_DO_BAD_PARAMS == pt_res) {
         fprintf(sg_warnings_strm, "%s: bad pass through setup\n", leadin);
         return -1;
-    } else if (SCSI_PT_DO_TIMEOUT == res) {
+    } else if (SCSI_PT_DO_TIMEOUT == pt_res) {
         fprintf(sg_warnings_strm, "%s: pass through timeout\n", leadin);
         return -1;
     }
     if ((verbose > 2) && ((duration = get_scsi_pt_duration_ms(ptvp)) >= 0))
         fprintf(sg_warnings_strm, "      duration=%d ms\n", duration);
-    resid = (mx_resp_len > 0) ? get_scsi_pt_resid(ptvp) : 0;
+    resid = (mx_di_len > 0) ? get_scsi_pt_resid(ptvp) : 0;
+    slen = get_scsi_pt_sense_len(ptvp);
     switch ((cat = get_scsi_pt_result_category(ptvp))) {
     case SCSI_PT_RESULT_GOOD:
-        if (mx_resp_len > 0) {
-            got = mx_resp_len - resid;
+        if (slen > 7) {
+            resp_code = sbp[0] & 0x7f;
+            /* SBC referrals can have status=GOOD and sense_key=COMPLETED */
+            if (resp_code >= 0x70) {
+                if (resp_code < 0x72) {
+                    if (SPC_SK_NO_SENSE != (0xf & sbp[2]))
+                        sg_err_category_sense(sbp, slen);
+                } else if (resp_code < 0x74) {
+                    if (SPC_SK_NO_SENSE != (0xf & sbp[1]))
+                        sg_err_category_sense(sbp, slen);
+                }
+            }
+        }
+        if (mx_di_len > 0) {
+            got = mx_di_len - resid;
             if (verbose && (resid > 0))
-                fprintf(sg_warnings_strm, "    %s: requested %d bytes but "
-                        "got %d bytes\n", leadin, mx_resp_len, got);
+                fprintf(sg_warnings_strm, "    %s: pass-through requested "
+                        "%d bytes but got %d bytes\n", leadin, mx_di_len,
+                        got);
             return got;
         } else
             return 0;
@@ -173,8 +174,7 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin, int res,
         }
         return -1;
     case SCSI_PT_RESULT_SENSE:
-        slen = get_scsi_pt_sense_len(ptvp);
-        scat = sg_err_category_sense(sense_b, slen);
+        scat = sg_err_category_sense(sbp, slen);
         switch (scat) {
         case SG_LIB_CAT_NOT_READY:
         case SG_LIB_CAT_UNIT_ATTENTION:
@@ -186,20 +186,21 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin, int res,
             break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_MEDIUM_HARD:
+            ++check_data_in;
         default:
             n = noisy;
             break;
         }
         if (verbose || n) {
-            sg_get_sense_str(leadin, sense_b, slen, (verbose > 1),
+            sg_get_sense_str(leadin, sbp, slen, (verbose > 1),
                              sizeof(b), b);
             fprintf(sg_warnings_strm, "%s", b);
-        }
-        if (verbose && (mx_resp_len > 0) && (resid > 0)) {
-            got = mx_resp_len - resid;
-            if ((verbose > 2) || (got > 0))
-                fprintf(sg_warnings_strm, "    requested %d bytes but "
-                        "got %d bytes\n", mx_resp_len, got);
+            if ((mx_di_len > 0) && (resid > 0)) {
+                got = mx_di_len - resid;
+                if ((verbose > 2) || check_data_in || (got > 0))
+                    fprintf(sg_warnings_strm, "    pass-through requested "
+                            "%d bytes but got %d bytes\n", mx_di_len, got);
+            }
         }
         if (o_sense_cat)
             *o_sense_cat = scat;
@@ -1338,8 +1339,14 @@ sg_ll_log_sense(int sg_fd, int ppc, int sp, int pc, int pg_code,
             ret = -1;
             break;
         }
-    } else
+    } else {
+        if ((mx_resp_len > 3) && (ret < 4)) {
+            /* resid indicates LOG SENSE response length bad, so zero it */
+            resp[2] = 0;
+            resp[3] = 0;
+        }
         ret = 0;
+    }
     destruct_scsi_pt_obj(ptvp);
     return ret;
 }
