@@ -1,30 +1,8 @@
 /*
- * Copyright (c) 2009 Douglas Gilbert.
+ * Copyright (c) 2009-2011 Douglas Gilbert.
  * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
+ * Use of this source code is governed by a BSD-style
+ * license that can be found in the BSD_LICENSE file.
  */
 
 #include <unistd.h>
@@ -48,7 +26,7 @@
 #include "sg_cmds_basic.h"
 #include "sg_cmds_extra.h"
 
-static char * version_str = "0.93 20090610";
+static char * version_str = "0.99 20111117";
 
 
 #define ME "sg_write_same: "
@@ -61,6 +39,7 @@ static char * version_str = "0.93 20090610";
 #define WRITE_SAME10_LEN 10
 #define WRITE_SAME16_LEN 16
 #define WRITE_SAME32_LEN 32
+#define RCAP10_RESP_LEN 8
 #define RCAP16_RESP_LEN 32
 #define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
 #define DEF_TIMEOUT_SECS 60
@@ -70,8 +49,10 @@ static char * version_str = "0.93 20090610";
 #define EBUFF_SZ 256
 
 static struct option long_options[] = {
+    {"10", no_argument, 0, 'R'},
     {"16", no_argument, 0, 'S'},
     {"32", no_argument, 0, 'T'},
+    {"anchor", no_argument, 0, 'a'},
     {"grpnum", required_argument, 0, 'g'},
     {"help", no_argument, 0, 'h'},
     {"in", required_argument, 0, 'i'},
@@ -89,6 +70,7 @@ static struct option long_options[] = {
 };
 
 struct opts_t {
+    int anchor;
     int grpnum;
     char ifilename[256];
     uint64_t lba;
@@ -101,6 +83,7 @@ struct opts_t {
     int wrprotect;
     int xfer_len;
     int pref_cdb_size;
+    int want_ws10;
 };
 
 
@@ -109,17 +92,21 @@ static void
 usage()
 {
   fprintf(stderr, "Usage: "
-          "sg_write_same [--16] [--32] [--grpnum=GN] [--help] [--in=IF]\n"
-          "                     [--lba=LBA] [--lbdata] [--num=NUM] "
-          "[--pbdata]\n"
+          "sg_write_same [--10] [--16] [--32] [--anchor] [--grpnum=GN] "
+          "[--help]\n"
+          "                     [--in=IF] [--lba=LBA] [--lbdata] "
+          "[--num=NUM] [--pbdata]\n"
           "                     [--timeout=TO] [--unmap] [--verbose] "
           "[--version]\n"
           "                     [--wrprotect=WRP] [xferlen=LEN] DEVICE\n"
           "  where:\n"
+          "    --10|-R              do WRITE SAME(10) (even if '--unmap' "
+          "is given)\n"
           "    --16|-S              do WRITE SAME(16) (def: 10 unless "
           "'--unmap' given\n"
           "                         or LBA+NUM needs more than 32 bits)\n"
           "    --32|-T              do WRITE SAME(32) (def: 10 or 16)\n"
+          "    --anchor|-a          set anchor field in cdb\n"
           "    --grpnum=GN|-g GN    GN is group number field (def: 0)\n"
           "    --help|-h            print out usage message\n"
           "    --in=IF|-i IF        IF is file to fetch one block of data "
@@ -162,7 +149,7 @@ do_write_same(int sg_fd, const struct opts_t * optsp, const void * dataoutp,
     if (WRITE_SAME10_LEN == cdb_len) {
         llba = optsp->lba + optsp->numblocks;
         if ((optsp->numblocks > 0xffff) || (llba > ULONG_MAX) ||
-            optsp->unmap) {
+            (optsp->unmap && (0 == optsp->want_ws10))) {
             cdb_len = WRITE_SAME16_LEN;
             if (optsp->verbose)
                 fprintf(stderr, "do_write_same: use WRITE SAME(16) instead "
@@ -176,6 +163,12 @@ do_write_same(int sg_fd, const struct opts_t * optsp, const void * dataoutp,
     case WRITE_SAME10_LEN:
         wsCmdBlk[0] = WRITE_SAME10_OP;
         wsCmdBlk[1] = ((optsp->wrprotect & 0x7) << 5);
+        /* ANCHOR + UNMAP not allowed for WRITE_SAME10 in sbc3r24+r25 but
+         * a proposal has been made to allow it. Anticipate approval. */
+        if (optsp->anchor)
+            wsCmdBlk[1] |= 0x10;
+        if (optsp->unmap)
+            wsCmdBlk[1] |= 0x8;
         if (optsp->pbdata)
             wsCmdBlk[1] |= 0x4;
         if (optsp->lbdata)
@@ -192,6 +185,8 @@ do_write_same(int sg_fd, const struct opts_t * optsp, const void * dataoutp,
     case WRITE_SAME16_LEN:
         wsCmdBlk[0] = WRITE_SAME16_OP;
         wsCmdBlk[1] = ((optsp->wrprotect & 0x7) << 5);
+        if (optsp->anchor)
+            wsCmdBlk[1] |= 0x10;
         if (optsp->unmap)
             wsCmdBlk[1] |= 0x8;
         if (optsp->pbdata)
@@ -217,6 +212,8 @@ do_write_same(int sg_fd, const struct opts_t * optsp, const void * dataoutp,
         wsCmdBlk[8] = ((WRITE_SAME32_SA >> 8) & 0xff);
         wsCmdBlk[9] = (WRITE_SAME32_SA & 0xff);
         wsCmdBlk[10] = ((optsp->wrprotect & 0x7) << 5);
+        if (optsp->anchor)
+            wsCmdBlk[10] |= 0x10;
         if (optsp->unmap)
             wsCmdBlk[10] |= 0x8;
         if (optsp->pbdata)
@@ -327,12 +324,15 @@ main(int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "g:hi:l:Ln:PSt:TUvVw:x:", long_options,
+        c = getopt_long(argc, argv, "ag:hi:l:Ln:PRSt:TUvVw:x:", long_options,
                         &option_index);
         if (c == -1)
             break;
 
         switch (c) {
+        case 'a':
+            ++opts.anchor;
+            break;
         case 'g':
             opts.grpnum = sg_get_num(optarg);
             if ((opts.grpnum < 0) || (opts.grpnum > 31))  {
@@ -371,7 +371,15 @@ main(int argc, char * argv[])
         case 'P':
             ++opts.pbdata;
             break;
+        case 'R':
+            ++opts.want_ws10;
+            break;
         case 'S':
+            if (DEF_WS_CDB_SIZE != opts.pref_cdb_size) {
+                fprintf(stderr, "only one '--10', '--16' or '--32' "
+                        "please\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
             opts.pref_cdb_size = 16;
             break;
         case 't':
@@ -382,6 +390,11 @@ main(int argc, char * argv[])
             }
             break;
         case 'T':
+            if (DEF_WS_CDB_SIZE != opts.pref_cdb_size) {
+                fprintf(stderr, "only one '--10', '--16' or '--32' "
+                        "please\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
             opts.pref_cdb_size = 32;
             break;
         case 'U':
@@ -426,6 +439,10 @@ main(int argc, char * argv[])
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+    if (opts.want_ws10 && (DEF_WS_CDB_SIZE != opts.pref_cdb_size)) {
+        fprintf(stderr, "only one '--10', '--16' or '--32' please\n");
+        return SG_LIB_SYNTAX_ERROR;
+    }
     if (NULL == device_name) {
         fprintf(stderr, "missing device name!\n");
         usage();
@@ -466,15 +483,38 @@ main(int argc, char * argv[])
         res = sg_ll_readcap_16(sg_fd, 0 /* pmi */, 0 /* llba */, resp_buff,
                                RCAP16_RESP_LEN, 0, (vb ? (vb - 1): 0));
         if (0 == res) {
+            if (vb > 3)
+                dStrHex((const char *)resp_buff, RCAP16_RESP_LEN, 1);
             block_size = ((resp_buff[8] << 24) |
                           (resp_buff[9] << 16) |
                           (resp_buff[10] << 8) |
                           resp_buff[11]);
             prot_en = !!(resp_buff[12] & 0x1);
-            opts.xfer_len = block_size + (prot_en ? 8 : 0);
+            opts.xfer_len = block_size;
+            if (prot_en && (opts.wrprotect > 0))
+                opts.xfer_len += 8;
+        } else if ((SG_LIB_CAT_INVALID_OP == res) ||
+                   (SG_LIB_CAT_ILLEGAL_REQ == res)) {
+            if (vb)
+                fprintf(stderr, "Read capacity(16) not supported, try Read "
+                        "capacity(10)\n");
+            res = sg_ll_readcap_10(sg_fd, 0 /* pmi */, 0 /* lba */, resp_buff,
+                                   RCAP10_RESP_LEN, 0, (vb ? (vb - 1): 0));
+            if (0 == res) {
+                if (vb > 3)
+                    dStrHex((const char *)resp_buff, RCAP10_RESP_LEN, 1);
+                block_size = ((resp_buff[4] << 24) |
+                              (resp_buff[5] << 16) |
+                              (resp_buff[6] << 8) |
+                              resp_buff[7]);
+                opts.xfer_len = block_size;
+            }
         } else if (vb)
             fprintf(stderr, "Read capacity(16) failed. Unable to calculate "
                     "block size\n");
+        if (res)
+            fprintf(stderr, "Read capacity(10) failed. Unable to calculate "
+                    "block size\n");
     }
     if (opts.xfer_len < 1) {
         fprintf(stderr, "unable to deduce block size, please give "
@@ -529,9 +569,10 @@ main(int argc, char * argv[])
             close(infd);
     } else {
         if (vb)
-            fprintf(stderr, "Default data-out buffer to %d zeroes\n",
+            fprintf(stderr, "Default data-out buffer set to %d zeros\n",
                     opts.xfer_len);
-        if (prot_en) { /* default for protection is 0xff, rest get 0x0 */
+        if (prot_en && (opts.wrprotect > 0)) {
+           /* default for protection is 0xff, rest get 0x0 */
             memset(wBuff + opts.xfer_len - 8, 0xff, 8);
             if (vb)
                 fprintf(stderr, " ... apart from last 8 bytes which are set "
